'use strict';

function extend(dest, src) {
  if (src) {
    var props = Object.keys(src)
    for (var i = 0, l = props.length; i < l ; i++) {
      dest[props[i]] = src[props[i]]
    }
  }
  return dest
}

function copy(obj) {
  return extend({}, obj)
}

// XXX Looks like the guts of a Pattern object?

var PLACEHOLDER = '_'

var ALPHANNUMERIC = '*'
var DIGIT = '1'
var LETTER = 'A'

var DIGIT_RE = /^\d$/
var LETTER_RE = /^[A-Za-z]$/
var ALPHANNUMERIC_RE = /^[\dA-Za-z]$/

function isFormatCharacter(char) {
  return char === DIGIT || char === LETTER || char === ALPHANNUMERIC
}

function isValidCharacter(char, format) {
  if (format === DIGIT) { return DIGIT_RE.test(char) }
  if (format === LETTER) { return LETTER_RE.test(char) }
  if (format === ALPHANNUMERIC) { return ALPHANNUMERIC_RE.test(char) }
  throw new Error('Unknown format character: ' + format)
}

function formatValueToPattern(value, pattern) {
  var valueBuffer = new Array(pattern.length)
  var valueIndex = 0
  for (var i = 0, l = pattern.length; i < l ; i++) {
    if (isFormatCharacter(pattern[i])) {
      valueBuffer[i] = value.length > valueIndex && isValidCharacter(value[valueIndex], pattern[i])
                       ? value[valueIndex]
                       : PLACEHOLDER
      valueIndex++
    }
    else {
      valueBuffer[i] = pattern[i]
      // Also allow the value to contain static values from the pattern by
      // advancing its index.
      if (value.length > valueIndex && value[valueIndex] === pattern[i]) {
        valueIndex++
      }
    }
  }
  return valueBuffer
}

function InputMask(options) {
  if (!(this instanceof InputMask)) { return new InputMask(options) }

  options = extend({
    pattern: null,
    selection: {start: 0, end: 0},
    value: ''
  }, options)

  if (options.pattern == null) {
    throw new Error('InputMask: you must provide a pattern.')
  }

  this.setPattern(options.pattern, options.value)
  this.setSelection(options.selection)
}

/**
 * Applies a single character of input based on the current selection.
 * @param {string} char
 * @return {boolean} true if a change has been made to value or selection as a
 *   result of the input, false otherwise.
 */
InputMask.prototype.input = function input(char) {
  // Ignore additional input if the cursor's at the end of the pattern
  if (this.selection.start === this.selection.end &&
      this.selection.start === this.pattern.length) {
    return false
  }

  var inputIndex = this.selection.start

  // If a range of characters was selected and it includes the first editable
  // character, make sure any input given is applied to it.
  if (this.selection.start !== this.selection.end &&
      this.selection.start < this._firstEditableIndex &&
      this.selection.end > this._firstEditableIndex) {
    inputIndex = this._firstEditableIndex
  }

  var format = this.pattern[inputIndex]

  // Bail out or add the character to input
  if (isFormatCharacter(format)) {
    if (!isValidCharacter(char, format)) {
      return false
    }
    this.value[inputIndex] = char
  }
  else {
    // The user must have placed the cursor prior to, or began a selection with,
    // a static part of the pattern, so skip over it.
    this.value[inputIndex] = format
  }

  // If multiple characters were selected, blank the remainder out based on the
  // pattern.
  var end = this.selection.end - 1
  while (end > inputIndex) {
    format = this.pattern[end]
    this.value[end] = isFormatCharacter(format) ? PLACEHOLDER : format
    end--
  }

  // Advance the cursor to the next character
  this.selection.start = this.selection.end = inputIndex + 1

  // Skip over any subsequent static characters
  while (this.pattern.length > this.selection.start &&
         !isFormatCharacter(this.pattern[this.selection.start])) {
    this.value[this.selection.start] = this.pattern[this.selection.start]
    this.selection.start++
    this.selection.end++
  }

  return true
}

/**
 * Attempts to delete from the value based on the current cursor position or
 * selection.
 * @return {boolean} true if the value or selection changed as the result of
 *   backspacing, false otherwise.
 */
InputMask.prototype.backspace = function backspace() {
  // If the cursor is at the start there's nothing to do
  if (this.selection.start === 0 && this.selection.end === 0) {
    return false
  }

  var format

  // No range selected - work on the character preceding the cursor
  if (this.selection.start === this.selection.end) {
    format = this.pattern[this.selection.start - 1]
    this.value[this.selection.start - 1] = isFormatCharacter(format) ? PLACEHOLDER : format
    this.selection.start--
    this.selection.end--
  }
  // Range selected - delete characters and leave the cursor at the start of the selection
  else {
    var end = this.selection.end - 1
    while (end >= this.selection.start) {
      format = this.pattern[end]
      this.value[end] = isFormatCharacter(format) ? PLACEHOLDER : format
      end--
    }
    this.selection.end = this.selection.start
  }

  return true
}

/**
 * Attempts to paste a string of input at the current cursor position or over
 * the top of the current selection.
 * Invalid content at any position will cause the paste to be rejected, and it
 * may contain static parts of the mask's pattern.
 * @param {string} input
 * @return {boolean} true if the paste was successful, false otherwise.
 */
InputMask.prototype.paste = function paste(input) {
  var initialValue = this.value.slice()
  var initialSelection = copy(this.selection)

  // If there are static characters at the start of the pattern and the cursor
  // or selection is within them, the static characters must match for a valid
  // paste.
  if (this.selection.start < this._firstEditableIndex) {
    for (var i = 0, l = this._firstEditableIndex - this.selection.start; i < l; i++) {
      if (input.charAt(i) !== this.pattern[i]) {
        return false
      }
    }

    // Continue as if the selection and input started from the editable part of
    // the pattern.
    input = input.substring(this._firstEditableIndex - this.selection.start)
    this.selection.start = this._firstEditableIndex
  }

  for (var i = 0, l = input.length;
       i < l && this.selection.start <= this._lastEditableIndex;
       i++) {
    var valid = this.input(input.charAt(i))
    // Allow static parts of the pattern to appear in pasted input - they will
    // already have been stepped over by input(), so verify that the value
    // deemed invalid by input() was the expected static character.
    if (!valid) {
      if (this.selection.start > 0) {
        // XXX This only allows for one static character to be skipped
        var format = this.pattern[this.selection.start - 1]
        if (!isFormatCharacter(format) && input.charAt(i) === format) {
          continue
        }
      }
      this.value = initialValue
      this.selection = initialSelection
      return false
    }
  }
  return true
}

InputMask.prototype.setPattern = function setPattern(pattern, value) {
  var patternChars = pattern.split('')
  var firstEditableIndex = null
  var lastEditableIndex = null
  for (var i = 0, l = patternChars.length; i < l; i++) {
    if (isFormatCharacter(patternChars[i])) {
      if (firstEditableIndex === null) {
        firstEditableIndex = i
      }
      lastEditableIndex = i
    }
  }
  if (firstEditableIndex === null && lastEditableIndex === null) {
    throw new Error(
      'InputMask: pattern "' + pattern + '" does not contain any editable characters.'
    )
  }
  this.pattern = patternChars
  this._firstEditableIndex = firstEditableIndex
  this._lastEditableIndex = lastEditableIndex
  this.setValue(value || '')
}

InputMask.prototype.setSelection = function setSelection(selection) {
  this.selection = copy(selection)
  if (this.selection.start === this.selection.end) {
    if (this.selection.start < this._firstEditableIndex) {
      this.selection.start = this.selection.end = this._firstEditableIndex
      return true
    }
    if (this.selection.end > this._lastEditableIndex + 1) {
      this.selection.start = this.selection.end = this._lastEditableIndex + 1
      return true
    }
  }
  return false
}

InputMask.prototype.getValue = function getValue() {
  return this.value.join('')
}

InputMask.prototype.setValue = function setValue(value) {
  this.value = formatValueToPattern(value.split(''), this.pattern)
}

InputMask.formatValueToPattern = formatValueToPattern

module.exports = InputMask